// SJPEGTexture.cpp: Implementierung der Klasse SJPEGTexture.
//
// Copyright by Andr Stein
// E-Mail: andre_stein@gmx.de, andre_stein@web.de
// http://www.steinsoft.net
//////////////////////////////////////////////////////////////////////

#include "SJPEGFile.h"
#include "SJPEGTexture.h"

//Swap function

void swapBGRtoRGB(char* buffer,int size)
{
	for (int i = 0;i < size; i += 3)
	{
		buffer[i] ^= buffer[i+2] ^=
		buffer[i] ^= buffer[i+2];
	}
}

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

SJPEGTexture::SJPEGTexture()
{

}

SJPEGTexture::~SJPEGTexture()
{

}

SJPEGTexture::SJPEGTexture(const char *filename, int minFilter, int magFilter)
{
	createTexture(filename,minFilter,magFilter);
}


///////////////////////////////
// Funktionen
///////////////////////////////

bool SJPEGTexture::createMipmap(const char *filename, int minFilter, int magFilter)
{
	bool noError;
	SJPEGFile jpegFile;

	noError = jpegFile.loadJpegFile(filename);

	if (noError)
	{
		swapBGRtoRGB((char*)jpegFile.getData(),jpegFile.getHeight()*
										  jpegFile.getWidth()*
										  3);

		glPixelStorei(GL_UNPACK_ALIGNMENT,4);
		
		glGenTextures(1,&texture);
		glBindTexture(GL_TEXTURE_2D,texture);

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,minFilter);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,magFilter);
		
		gluBuild2DMipmaps(GL_TEXTURE_2D,3,jpegFile.getWidth(),
			jpegFile.getHeight(),GL_RGB,
			GL_UNSIGNED_BYTE,jpegFile.getData());

		return true;
	}
	
	return false;
}

bool SJPEGTexture::createTexture(const char *filename, int minFilter, int magFilter)
{
	bool noError;
	SJPEGFile jpegFile;

	noError = jpegFile.loadJpegFile(filename);

	if (noError)
	{
		swapBGRtoRGB((char*)jpegFile.getData(),jpegFile.getHeight()*
										  jpegFile.getWidth()*
										  3);

		glPixelStorei(GL_UNPACK_ALIGNMENT,4);
		
		glGenTextures(1,&texture);
		glBindTexture(GL_TEXTURE_2D,texture);

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,minFilter);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,magFilter);
		
		glTexImage2D(GL_TEXTURE_2D,0,3,jpegFile.getWidth(),
			jpegFile.getHeight(),0,GL_RGB,
			GL_UNSIGNED_BYTE,jpegFile.getData());
		
		return true;
	}

	
	return false;
}
